var env = process.env.NODE_ENV || 'development'; 
var cfg = require('../../config/config.js').get(env);
var stats = require('../helper/stats.js'); 
var request = require('request');
var messageSOAP = require('../helper/messageSOAP.js'); 
var log = require('../helper/log.js');
var constants = require('../helper/constants.js');
var moment = require('moment');

var connection = [];


connection.requestSoapToSACF = async function (req,soap,cmd,myMethod,headers) {
    try {
        log.log("Connect to SACF")

        var service = cfg.service.SACF;
        var retry = service[myMethod+"_"+cmd+"_Retry"];    

        var params = {
            url                 : service[myMethod+"_"+cmd+"_URL"],
            headers             : headers,
            body                : messageSOAP.objectToSOAP(soap,cmd),
            method              : myMethod,
            timeout             : cfg.service.SACF[myMethod+"_"+cmd+"_Timeout"]*1000
        }

        log.log("requestData : "+JSON.stringify(params))

        if(!retry)                
            retry = cfg.Default_Retry;

        var objectData = {
            maxRetry : retry,
            curRetry : 1
        }
        
        
        
        var funStats = {
            start : function(){
                stats.sendSACFRequest(service.Name,cmd);
                log.logDetail.addOutput(req,service.Name,myMethod+"_"+cmd,constants.REQUEST,params,soap);
            },
            end :  function(resultObj){
                
                var returnData = {};

                if(resultObj.err)
                {
                    //connect err => retry
                    var response = {};
                    if(resultObj.err.code == constants.RESPONSECONN.STATS.ETIMEDOUT){
                        response = constants.RESPONSECONN.MESSAGE.ETIMEDOUT;
                    } else if(resultObj.err.code == constants.RESPONSECONN.STATS.ECONNREFUSED){
                        response = constants.RESPONSECONN.MESSAGE.ECONNREFUSED;
                    } else if(resultObj.err.code == constants.RESPONSECONN.STATS.ESOCKETTIMEDOUT){
                        response = constants.RESPONSECONN.MESSAGE.ESOCKETTIMEDOUT;
                    }

                    stats.receiveSACFResponse(service.Name,cmd,constants.ERROR);                    
                    log.logDetail.addInput(req,service.Name,myMethod+"_"+cmd,constants.RESPONSE,resultObj,response);
                    log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,response.resultCode,response.resultDescription);

                    returnData.retry = true;
                    returnData.response = response;
                    returnData.err =  constants.RESPONSERESULT.ERROR.developerMessage;
                }else
                {
                    if(!resultObj.response)
                    {
                        //no response data => retry
                        var response = constants.RESPONSERESULT.ERROR;
                        stats.receiveSACFBadResponse(service.Name,cmd);                    
                        log.logDetail.addInput(req,service.Name,myMethod+"_"+cmd,constants.RESPONSE,resultObj,response);
                        log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,response.resultCode,response.resultDescription);

                        returnData.retry = true;
                        returnData.response = response;
                        returnData.err =  constants.RESPONSERESULT.ERROR.developerMessage;
                    }else
                    {
                        returnData.retry = false;
                        returnData.response = resultObj;
        
                        
                    }
                }

                return returnData;
               
            },
            // error : function(response){log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,response.resultCode,response.resultDescription);}
        };


        var result = await asyncRequest(params,objectData,funStats);

        // return checkResponse(result,funStats);
        return result;
    }
    catch(err) {
        log.log(err);

        var result = {
            response : constants.RESPONSECONN.MESSAGE.ERROR
        };
        return result;

    }
};

connection.requestJsonToD01 = async function (req,obj,cmd,myMethod,filter) {
    try {
    
        log.log("Connect to D01")

        if(myMethod === constants.METHOD.POST)
            obj.dateCreate = moment().format("YYYY-MM-DDTHH:mm:ss+0700");

        var service = cfg.service.D01;    
        var retry = service[myMethod+"_"+cmd+"_Retry"];
        if(!retry)                
            retry = cfg.Default_Retry;

        var objectData = {
            maxRetry : retry,
            curRetry : 1
        }

        // JSON.stringify(json)
        var params = {
            url                 : service[myMethod+"_"+cmd+"_URL"],
            body                : obj,
            method              : myMethod,
            timeout             : service[myMethod+"_"+cmd+"_Timeout"]*1000
        }

        log.log("requestData : "+JSON.stringify(params))

        var funStats = {
            start : function(){
                stats.sendRestRequest(service.Name,myMethod,cmd);
                log.logDetail.addOutput(req,service.Name,myMethod+"_"+cmd,constants.REQUEST,params,obj);
            },
            end : function(resultObj){

                var returnData = {};

                if(resultObj.err)
                {
                    //connect err => retry
                    var response = {};
                    if(resultObj.err.code == constants.RESPONSECONN.STATS.ETIMEDOUT){
                        response = constants.RESPONSECONN.MESSAGE.ETIMEDOUT;
                    } else if(resultObj.err.code == constants.RESPONSECONN.STATS.ECONNREFUSED){
                        response = constants.RESPONSECONN.MESSAGE.ECONNREFUSED;
                    } else if(resultObj.err.code == constants.RESPONSECONN.STATS.ESOCKETTIMEDOUT){
                        response = constants.RESPONSECONN.MESSAGE.ESOCKETTIMEDOUT;
                    }

                    stats.receiveRestResponse(service.Name,myMethod,cmd,constants.ERROR);                    
                    log.logDetail.addInput(req,service.Name,myMethod+"_"+cmd,constants.RESPONSE,resultObj,response);
                    log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,response.resultCode,response.resultDescription);

                    returnData.retry = true;
                    returnData.response = response;
                    returnData.err =  constants.RESPONSERESULT.ERROR.developerMessage;
                }else
                {
                    if(!resultObj.response)
                    {
                        //no response data => retry
                        var response = constants.RESPONSERESULT.ERROR;
                        stats.receiveBadRestResponse(service.Name,myMethod,cmd);                    
                        log.logDetail.addInput(req,service.Name,myMethod+"_"+cmd,constants.RESPONSE,resultObj,response);
                        log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,response.resultCode,response.resultDescription);

                        returnData.retry = true;
                        returnData.response = response;
                        returnData.err =  constants.RESPONSERESULT.ERROR.developerMessage;
                    }else
                    {
                        try
                        {
                            var result = JSON.parse(resultObj.body);

                            log.log('ResponseData : ' + resultObj.body);
                        
                            if(!result.resultCode || !result.resultDescription)
                            {
                                //Missing resultCode or resultDescription => retry
                                log.log("Missing resultCode or resultDescription");
                                var response = result;
                                stats.receiveBadRestResponse(service.Name,myMethod,cmd);                    
                                log.logDetail.addInput(req,service.Name,myMethod+"_"+cmd,constants.RESPONSE,resultObj,response);
                                log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,response.resultCode,response.resultDescription);

                                returnData.retry = true;
                                returnData.response = response;
                                returnData.err =  constants.RESPONSERESULT.MISSING_INVALID.developerMessage;
                            
                            }else
                            {
                                //check normal
                                var response = result;
                                log.logDetail.addInput(req,service.Name,myMethod+"_"+cmd,constants.RESPONSE,resultObj,result);
                                if(result.resultCode.startsWith("2"))
                                {
                                    if(myMethod == constants.METHOD.GET){
                                        if(result.resultData && result.resultData.length>0)
                                        {
                                            stats.receiveRestResponse(service.Name,myMethod,cmd,constants.SUCCESS);
                                            log.addSuccessSummary(req,service.Name,myMethod+"_"+cmd,result.resultCode,constants.SUCCESS);
                                        }else
                                        {
                                            stats.receiveRestResponse(service.Name,myMethod,cmd,constants.RESPONSERESULT.DATA_NOT_FOUND.developerMessage);
                                            log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,constants.RESPONSERESULT.DATA_NOT_FOUND.resultCode,constants.RESPONSERESULT.DATA_NOT_FOUND.developerMessage);
                                        }
                                    } else {
                                        stats.receiveRestResponse(service.Name,myMethod,cmd,constants.SUCCESS);
                                        log.addSuccessSummary(req,service.Name,myMethod+"_"+cmd,result.resultCode,constants.SUCCESS);
                                    }

                                    returnData.retry = false;
                                    returnData.response = response;
                                    // returnData.err =  response.resultDescription;
                                } else if(result.resultCode.startsWith("404")){
                                    stats.receiveRestResponse(service.Name,myMethod,cmd,constants.RESPONSERESULT.DATA_NOT_FOUND.developerMessage);
                                    log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,result.resultCode,result.developerMessage);

                                    returnData.retry = false;
                                    returnData.response = response;
                                    // returnData.err =  response.resultDescription;
                                } else if(result.resultCode.startsWith("5")){
                                    stats.receiveRestResponse(service.Name,myMethod,cmd,constants.ERROR);
                                    log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,result.resultCode,result.developerMessage);
                                    //retry

                                    returnData.retry = true;
                                    returnData.response = response;
                                    // returnData.err =  response.resultDescription;
                                } else {
                                    stats.receiveRestResponse(service.Name,myMethod,cmd,constants.ERROR);
                                    log.addErrorSummary(req,service.Name,myMethod+"_"+cmd,result.resultCode,result.developerMessage);
                                    //retry
                                    
                                    returnData.retry = true;
                                    returnData.response = response;
                                    // returnData.err =  response.resultDescription;
                                }
                            }
                        }catch(err) {

                            // returnData.response = constants.RESPONSERESULT.ERROR;
                            returnData.err =  constants.RESPONSERESULT.ERROR;
                            returnData.retry = true;
                    
                        }

                    }
                }

                return returnData;

            }
        };

        

        var result = {};
        result.response = {};
        var resultData = [];        

        if(params.method == constants.METHOD.GET)
        {
            if(filter && filter == "or"){
                var genUrl = getOrUrl(params);
            }
            else{
                var genUrl = getUrl(params,service["searchRecordLimit"]);   
            }

            if(genUrl.length > 1)
            {
                console.log("multi Send");
                var multiSend = [];
                for(var i=0;i<genUrl.length;i++)
                    multiSend.push({url:genUrl[i],method:params.method});
            
                var parallelRes = await parallelRequest(multiSend,objectData,funStats); 

                var dataRes = {};
                for(var i=0;i<multiSend.length;i++)
                {
                    // console.log("start loop");
                    var loopRes = parallelRes[i];
                    
                    if(loopRes.response)
                    {
                        // var data = JSON.parse(loopRes.response);
                        var data = loopRes.response;
                        dataRes[i] = {};

                        if(!data.resultCode)
                            dataRes[i].err = "Miss resultCode";

                        if(!data.resultDescription)
                            dataRes[i].err = "Miss resultDescription"

                        
                           
                        dataRes[i].data = data.resultData;
                        // console.log(dataRes[i].data);
                        
                    }

                }

                for (var i = 0; i < multiSend.length; i++) {
                        
                    resultData = resultData.concat(dataRes[i].data);
                    if(dataRes[i].err)
                        result.err = dataRes[i].err;

                        
                }

                if(!result.err)
                {
                    result.response = constants.RESPONSECONN.MESSAGE.SUCCESS;
                    result.response.resultData = resultData;
                    // result.response.resultCode = "20000";
                    // result.response.developerMessage = "Success";
                }

            }else
            {
                // console.log("single Send");
                params.url = genUrl[0];
                delete params.body;

                result = await asyncRequest(params,objectData,funStats);
                // var resultRes = await asyncRequest(params,objectData,funStats);

                // // console.log(resultRes);
                // if(resultRes.response)
                // {
                //     var data = JSON.parse(resultRes.response.body);
                //     result.response = data;
                
                // } else {
                //     result = resultRes;
                // }
            }

            
        } else if(params.method == constants.METHOD.POST) {

            params.body = JSON.stringify(params.body);
            result = await asyncRequest(params,objectData,funStats);
            // var resultRes = await asyncRequest(params,objectData,funStats);

            // if(resultRes.response){
            //     result.response = JSON.parse(resultRes.body);
            // } else {
            //     result = resultRes;
            // }
        } else if(params.method == constants.METHOD.PUT) {

            params.url = getUrlFilter(params.url,filter);

            params.body = JSON.stringify(params.body);


            result = await asyncRequest(params,objectData,funStats);

            // var resultRes = await asyncRequest(params,objectData,funStats);
            
            // if(resultRes.response){
            //     result.response = JSON.parse(resultRes.body);
            // } else {
            //     result = resultRes;
            // }
        }

        // console.log(result);
        
        // if(result.response)
        // {
        //     if(!result.response.resultCode){
        //         result.err = "Missing resultCode";
        //     } else if(!result.response.resultDescription){
        //         result.err = "Missing resultDescription";
        //     }
        // }

        // return checkResponse(result,funStats);

        return result;

    }
    catch(err) {
        log.log(err);

        var result = {
            response : constants.RESPONSECONN.MESSAGE.ERROR
        };
        return result;

    }
};



async function parallelRequest(multiParams,objectData,funStat)
{
   
    var promises = [];

    for(var i=0;i<multiParams.length;i++)
        promises.push(asyncRequest (multiParams[i],objectData,funStat));

    var res = await Promise.all(promises);
    // for(var i=0;i<res.length;i++)
    // {
    //     if(!res[i].err)
    //     {
    //         console.log(res[i].body);
    //     }
    //     // stats.receiveResponse(node,cmd,"Success");
    

    // }

    

    return res;
}

function asyncRequest (params = {},objectData,funStat) {

    // console.log(objectData);  
   

    // console.log("Start : "+new Date().toISOString());
    return new Promise((resolve, reject) => {
        request(params, function (error, response, body) {

            funStat.start();
            log.log("URL : "+params.url);

            if (error) {

                // if(objectData.maxRetry == objectData.curRetry-1)
                // {
                    
                //     resolve({
                //         'body'  : body,
                //         'err'   : error
                //     });
                // }else
                // {                   
                //     var responseObj = {
                //         'body'  : body,
                //         'err'   : error
                //     };  
                       
                   
                //     if(funStat.end(responseObj))
                //     {
                //         objectData.curRetry++; 
                //         resolve(asyncRequest(params,objectData,funStat));
                //     }
                    
                // }
                
                
               
            } else {               
                

                // var responseObj = {
                //     'body'  : body,
                //     'response'   : response
                // };
                // resolve(funStat.end(responseObj));
            }

            var responseObj = {
                'body'  : body,
                'response'   : response,
                'err'   : error
            };

            var result = funStat.end(responseObj);

            if(result.retry)
            {
                if(objectData.maxRetry == objectData.curRetry-1)
                    resolve(result);
                else
                {
                    objectData.curRetry++;
                    resolve(asyncRequest(params,objectData,funStat));
                }
                    
            }else
                resolve(result);
            
        });
    });
};

function getUrlFilter(url,filter){
    var returnData = [];
    var myfilter = ""

    for(var i in filter){
        myfilter += "("+i+"="+filter[i]+")"
    }

    var myUrl = url+"?filter=(&"+myfilter+")"
    
    // var myUrl = url+"?filter=("+filter.base+")";

    returnData.push(myUrl);

    return myUrl;
}

function getUrl(params,limit)
{
    var returnData = [];
    var url = params.url;
    var fields = "";
    var filter = {
        base : "",
        multi : []
    };
    
    
    // params.body = {};
    // params.body.a = "aaa";
    // params.body.b = "bbb";
    // params.body.papa = [];
    // params.body.papa.push("1");
    // params.body.papa.push("2");
    // params.body.commandId = [];
    // params.body.commandId.push("1");
    // params.body.commandId.push("2");
    // params.body.commandId.push("3");
    // params.body.commandId.push("4");
    // params.body.commandId.push("5");
    // params.body.commandId.push("6");
    // params.body.commandId.push("7");
    // params.body.commandId.push("8");    
    // params.body.c = "ccc";
    // params.body.fields = "a,b,c,d";
    // params.body.customerId = ["1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6","1b6a768e52e9a1e11518a1c6"];
    // limit = 100;
    // console.log(params.body);  
    if(params.body)
    {
        var first = true;
        for (key in params.body) {

            if(key == "fields")
                fields = getFields(params.body.fields,first);
            else
                filter = getFilter(filter,key,params.body[key]);

            first = false;
        }

        if(first)
            fields += "?limit="+limit;
        else
            fields += "&limit="+limit;
        
    }   

    if(filter.multi.length > 0)
    {
        //have multi

        var careData = filter.multi[0];
        // console.log(freeSpace);

        for(var i=0;i<filter.multi.length;i++)
        {
            if(filter.multi[i].length > careData.length)
            {
                filter.base = filter.base+"(|"+careData.data+")";
                careData = filter.multi[i];
                
            }
        }


        var myUrl = url+"?filter=:"+filter.base+":"+fields;
        var freeSpace = cfg.Uri_length - myUrl.length;
        

        var arr = careData.data.split(")(");
            
        for(var j=0;j<arr.length;j++)
        {
            if(j==0)
                arr[j] = arr[j].substring(1);
            if(j==arr.length-1)
                arr[j] = arr[j].substring(0,arr[j].length-1);

            arr[j] = "("+arr[j]+")";
                // returnData.push(url+"?filter=("+newBasefilter+"(|"+arr[j]+")"+fields);
        }

        returnData = arrayUrl(returnData,url,filter.base,fields,freeSpace,arr);



    }else
    {
        var myUrl = url+"?filter=("+filter.base+")"+fields;
        returnData.push(myUrl);
    }

    // console.log(url);
    // console.log(filter);
    // console.log(fields);

    // for(var i=0;i<returnData.length;i++)
    //     log.log("URL "+[i+1]+" : "+returnData[i]);

    return returnData;
}

function getOrUrl(params){
    var returnData = [];
    var url = params.url;
    var filter = "";

    for(var i in params.body){
        filter = filter + "(" +i+ "=" +params.body[i]+ ")"
    }

    url = url + "filter(|" + filter + ")";

    returnData.push(url);

    return returnData;
}

function getFields(fields,first)
{
    var returnData;
    var prefix;
    if(first)
        prefix = "?fields=";
    else
        prefix = "&fields=";

    returnData = prefix+fields;

    return returnData;
}

function getFilter(filter,key,data)
{
   
    // console.log(key);
    // console.log(newfilter);
    if(!Array.isArray(data))
    {

        // var newfilter = {
        //     data : "("+key+"="+data+")",
        //     type : "single",
        //     length : ("("+key+"="+data+")").length
        // };

        if(filter.base != "")    
            filter.base = filter.base+"("+key+"="+data+")";
        else
            filter.base = "&"+filter.base+"("+key+"="+data+")";
    }else
    {
        var line = "";
        for(var i=0;i<data.length;i++)
        {
            line += "("+key+"="+data[i]+")";
        }

        var newfilter = {
            data : line,
            length : line.length
        };
    }

    if(newfilter)
        filter.multi.push(newfilter);
    return filter;
}

function arrayUrl(arrUrl,url,filter,fields,freeSpace,arrData)
{
    var loopUrl = "";
    var loopSpace = freeSpace;
    for(var i=0;i<arrData.length;i++)
    {
        if(loopSpace > (arrData[i].length+3))
        {
            loopUrl = loopUrl + arrData[i];
            loopSpace = loopSpace - arrData[i].length;
        }else
        {
            if(loopUrl != "")
            {
                arrUrl.push(url+"?filter=("+filter+"(|"+loopUrl+"))"+fields);

                loopUrl = arrData[i];
                loopSpace = freeSpace - arrData[i].length;
            }
            
        }

    }

    if(loopUrl != "")
        arrUrl.push(url+"?filter=("+filter+"(|"+loopUrl+"))"+fields);
    return arrUrl;
}

// function checkResponse(result,funStats)
// {
//     if(result.err)
//     {
//         log.log("Final Response : "+result.err,'error');
//         if(result.err.code == constants.RESPONSECONN.STATS.ETIMEDOUT){
//             result.response = constants.RESPONSECONN.MESSAGE.ETIMEDOUT;
//         } else if(result.err.code == constants.RESPONSECONN.STATS.ECONNREFUSED){
//             result.response = constants.RESPONSECONN.MESSAGE.ECONNREFUSED;
//         } else if(result.err.code == constants.RESPONSECONN.STATS.ESOCKETTIMEDOUT){
//             result.response = constants.RESPONSECONN.MESSAGE.ESOCKETTIMEDOUT;
//         } else {
//             if(result.err.resultDescription && result.err.resultDescription.indexOf(constants.RESPONSECONN.STATS.MISSING) != -1){
//                 result.response.resultCode = constants.RESPONSERESULT.MISSING_INVALID.resultCode;
//                 result.response.resultDescription = result.err;
//             } else {
//                 result.response = constants.RESPONSECONN.MESSAGE.ERROR;
//             }
//         }
//         funStats.error(result.response)
//     }else
//        log.log("Final Response : "+JSON.stringify(result));

//     return result;
// }

module.exports = connection;